package com.qmth.boot.core.solar.service;

import com.qmth.boot.core.solar.api.SolarApiClient;
import com.qmth.boot.core.solar.config.SolarProperties;
import com.qmth.boot.core.solar.crypto.AppLicenseUtil;
import com.qmth.boot.core.solar.model.AppControl;
import com.qmth.boot.core.solar.model.AppInfo;
import com.qmth.boot.core.solar.model.AppLicense;
import com.qmth.boot.core.solar.model.OrgInfo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.LinkedList;
import java.util.List;

@Component
public class SolarService {

    private static final Logger log = LoggerFactory.getLogger(SolarService.class);

    private SolarProperties solarProperties;

    private SolarApiClient solarApiClient;

    private AppLicense license;

    private AppInfo appInfo;

    public SolarService(SolarProperties solarProperties, SolarApiClient solarApiClient) {
        this.solarProperties = solarProperties;
        this.solarApiClient = solarApiClient;
        checkup();
    }

    /**
     * 获取APP基本信息
     *
     * @return
     */
    public AppInfo getAppInfo() {
        return appInfo;
    }

    /**
     * 获取APP控制参数
     *
     * @return
     */
    public AppControl getAppControl() {
        return appInfo != null ? appInfo.getControl() : null;
    }

    /**
     * 获取APP可以访问的机构集合，自动判断在线/离线激活两种模式
     *
     * @return
     */
    public List<OrgInfo> getOrgList() {
        if (license != null) {
            //offline
            return license.getOrgs();
        } else {
            //online
            List<OrgInfo> list = new LinkedList<>();
            int pageNumber = 1;
            int pageSize = 20;
            while (true) {
                List<OrgInfo> page = solarApiClient.listOrgInfo(pageNumber, pageSize);
                if (page == null || page.isEmpty()) {
                    break;
                }
                list.addAll(page);
                pageNumber++;
            }
            return list;
        }
    }

    /**
     * 使用AK信息在线激活
     *
     * @param accessKey
     * @param accessSecret
     * @return
     */
    public synchronized AppInfo update(String accessKey, String accessSecret) {
        appInfo = null;
        license = null;
        solarProperties.setAccessKey(accessKey);
        solarProperties.setAccessSecret(accessSecret);
        solarProperties.setLicense(null);
        try {
            appInfo = solarApiClient.getAppInfo();
        } catch (Exception e) {
            log.error("Solar api invoke error", e);
        }
        return appInfo;
    }

    /**
     * 使用授权文件离线激活，同时需要在有效期内
     *
     * @param licenseData
     * @return
     */
    public synchronized AppInfo update(byte[] licenseData) {
        appInfo = null;
        license = null;
        solarProperties.setAccessKey(null);
        solarProperties.setAccessSecret(null);
        AppLicense result = AppLicenseUtil.parseLicense(licenseData);
        if (result != null && !result.getControl().hasExpired()) {
            license = result;
            appInfo = result;
        }
        return appInfo;
    }

    private void checkup() {
        log.info("Solar startup checking...");
        if (StringUtils.isNotBlank(solarProperties.getAccessKey()) && StringUtils
                .isNotBlank(solarProperties.getAccessSecret())) {
            try {
                appInfo = solarApiClient.getAppInfo();
            } catch (Exception e) {
                log.error("Solar api invoke error", e);
            }
        } else if (StringUtils.isNotBlank(solarProperties.getLicense())) {
            //启动授权文件默认绑定硬件信息
            AppLicense license = AppLicenseUtil.parseLicense(solarProperties.getLicense());
            if (license != null) {
                if (license.getControl().hasExpired()) {
                    log.error("Solar license has expired");
                } else {
                    this.license = license;
                    this.appInfo = license;
                }
            }
        } else {
            log.warn("Solar startup check exit, accessKey&accessSecret and license file not exist!");
        }
        if (appInfo != null) {
            log.info("Solar startup check success, current app name: " + appInfo.getName());
        } else {
            log.info("Solar startup check faile, app unexist!");
        }
    }

}
